<?php defined('SYSPATH') or die('No direct access allowed.');
/**
 * Base class for service listing widgets
 *
 * @package   NINJA
 * @author    op5 AB
 * @license   GPL
 */
class tac_services_Base extends widget_Base {
	protected $duplicatable = true;

	public function __construct($model) {
		parent::__construct($model);
		$service_group_list = Servicegroup_Model::get_all();
		$this->services_groups = array(
			'all' => _('All'),
			'' => ''
		);
		if ($service_group_list !== false) {
			foreach($service_group_list as $group)
				$this->services_groups[$group->servicegroup_name] = "{$group->alias} ({$group->servicegroup_name})";
		}

		$contact = Contact_Model::get_contact();
		if (!empty($contact)) {
			$contact = $contact->current();
			$this->cmd_ok = $contact->can_submit_commands;
		}
		unset($contact);

		$auth = Nagios_auth_Model::instance();
		$this->cmd_host_ok = $auth->command_hosts_root;
		$this->cmd_svc_ok = $auth->command_services_root;
		unset($auth);
	}

	/**
	 * Create header links for status listing
	 */
	private function header_links(
			$type='host',
			$filter_object='all',
			$title=false,
			$method=false,
			$sort_field_db=false,
			$sort_field_str=false,
			$host_status=false,
			$service_status=false,
			$service_props=false,
			$host_props=false)
	{

		$type = trim($type);
		if (!is_string($filter_object))
			$filter_object = 'all';
		$filter_object = trim($filter_object);
		$title = trim($title);
		if (empty($type) || empty($title))  {
			return false;
		}
		$header = false;
		$lable_ascending = _('ascending');
		$lable_descending = _('descending');
		$lable_sort_by = _('Sort by');
		$lable_last = _('last');
		switch ($type) {
			case 'host':
				$header['title'] = $title;
				if (!empty($method) &&!empty($filter_object) && !empty($sort_field_db)) {
					$header['url_asc'] = Router::$controller.'/'.$method.'/'.$filter_object.'?hoststatustypes='.$host_status.'&sort_order='.nagstat::SORT_ASC.'&sort_field='.$sort_field_db;
					$header['alt_asc'] = $lable_sort_by.' '.$lable_last.' '.$sort_field_str.' ('.$lable_ascending.')';
					$header['img_asc'] = $this->img_sort_up;
					$header['url_desc'] = Router::$controller.'/'.$method.'/'.$filter_object.'?hoststatustypes='.$host_status.'&sort_order='.nagstat::SORT_DESC.'&sort_field='.$sort_field_db;
					$header['img_desc'] = $this->img_sort_down;
					$header['alt_desc'] = $lable_sort_by.' '.$sort_field_str.' ('.$lable_descending.')';
				}
				break;
			case 'service':
				$header['title'] = $title;
				if (!empty($method) &&!empty($filter_object) && !empty($sort_field_db)) {
					$header['url_asc'] = $this->model->name.'_'.$this->model->instance_id.'_reorder(\''.$sort_field_db.'\', \''.nagstat::SORT_ASC.'\')';
					$header['img_asc'] = $this->img_sort_up;
					$header['alt_asc'] = $lable_sort_by.' '.$lable_last.' '.$sort_field_str.' ('.$lable_ascending.')';
					$header['url_desc'] = $this->model->name.'_'.$this->model->instance_id.'_reorder(\''.$sort_field_db.'\', \''.nagstat::SORT_DESC.'\')';
					$header['img_desc'] = $this->img_sort_down;
					$header['alt_desc'] = $lable_sort_by.' '.$sort_field_str.' ('.$lable_descending.')';
				}
				break;
		}
		return $header;
	}

	/**
	 * List status details for hosts and services
	 *
	 * @param $name
	 * @param $servicestatustypes
	 * @param $hoststatustypes
	 * @param $sort_order
	 * @param $sort_field
	 * @param $group_type
	 */
	protected function service($name='all', $hoststatustypes=false, $servicestatustypes=false, $service_props=false, $sort_order='ASC', $sort_field='host_name', $group_type=false, $hostprops=false)
	{
		$hoststatustypes = urldecode($hoststatustypes);
		$servicestatustypes = urldecode($servicestatustypes);
		$service_props = urldecode($service_props);
		$hostprops = urldecode($hostprops);
		$sort_order = urldecode($sort_order);
		$sort_field = urldecode($sort_field);
		$group_type = urldecode($group_type);
		$noheader = urldecode(false);
		$group_type = strtolower($group_type);

		if (!is_string($name))
			$name = 'all';
		$name = trim($name);
		$hoststatustypes = strtolower($hoststatustypes);
		$hoststatustypes = $hoststatustypes ==='false' || $hoststatustypes ==='true' ? false : $hoststatustypes;

		$servicestatustypes = strtolower($servicestatustypes);
		$servicestatustypes = $servicestatustypes ==='false' || $servicestatustypes==='true' ? false : $servicestatustypes;

		$sort_order = $sort_order == 'false' || empty($sort_order) ? 'ASC' : $sort_order;
		$sort_field = $sort_field == 'false' || empty($sort_field) ? 'host_name' : $sort_field;
		# handle host- or servicegroup details
		$host_model = new Host_Model();
		$host_model->show_services = true;
		$host_model->state_filter = $hoststatustypes;
		$host_model->service_filter = $servicestatustypes;
		$host_model->set_sort_field($sort_field);
		$host_model->set_sort_order($sort_order);
		$host_model->serviceprops = $service_props;
		$host_model->hostprops = $hostprops;

		if (!empty($group_type)) {
			# convert 'servicegroup' to 'service' and 'hostgroup' to 'host'
			$grouptype = str_replace('group', '', $group_type);

			# servicegroups should only show services in the group
			if ($group_type == 'servicegroup') {
				$result = Group_Model::get_group_info($grouptype, $name, $hoststatustypes, $servicestatustypes, $service_props, $hostprops);
				$tot = $result !== false ? count($result) : 0;
				unset($result);
				$result = Group_Model::get_group_info($grouptype, $name, $hoststatustypes,
					$servicestatustypes, $service_props, $hostprops, false, $sort_field, $sort_order);
			} else {
				$hostlist = Group_Model::get_group_hoststatus($grouptype, $name, $hoststatustypes, $servicestatustypes);
				$group_hosts = false;
				foreach ($hostlist as $host_info) {
					$group_hosts[] = $host_info->host_name;
				}
				$host_model->num_per_page = false;
				$host_model->offset = false;
				$host_model->count = true;

				$host_model->set_host_list($group_hosts);
				$result_cnt = $host_model->get_host_status();

				$tot = $result_cnt !== false ? $result_cnt : 0;
				$host_model->count = false;

				$host_model->set_host_list($group_hosts);
				$host_model->set_sort_field($sort_field);
				$host_model->set_sort_order($sort_order);

				$result = $host_model->get_host_status();
			}
		} else {
			$host_model->num_per_page = false;
			$host_model->offset = false;
			$host_model->count = true;

			$host_model->set_host_list($name);
			$result_cnt = $host_model->get_host_status();
			$tot = $result_cnt !== false ? $result_cnt : 0;
			$host_model->count = false;

			$host_model->set_sort_field($sort_field);
			$host_model->set_sort_order($sort_order);

			$host_model->set_host_list($name);
			$result = $host_model->get_host_status();
		}

		if (is_object($result))
			$result = $result->as_array();

		return $result;
	}

	public function options()
	{
		$options = parent::options();
		$opt = new option($this->model->name, 'service_box_height', 'Height (px)', 'input', array(), 0);
		$opt->should_render_js(false);
		$options[] = $opt;
		$options[] = new option($this->model->name, 'critical', 'Show critical', 'checkbox', array(), true);
		$options[] = new option($this->model->name, 'warning', 'Show warning', 'checkbox', array(), false);
		$options[] = new option($this->model->name, 'unknown', 'Show unknown', 'checkbox', array(), false);
		$options[] = new option($this->model->name, 'group', 'Service group', 'dropdown', array('options' => $this->services_groups), 'all');
		return $options;
	}

	protected function setup() {
		$this->arguments = $this->get_arguments();
		if (isset($_GET['sort_order'])) {
			$this->order = $_GET['sort_order'];
			$this->model->merge_settings(array('sort_order' => $this->order));
		} elseif (isset($this->arguments['sort_order'])) {
			$this->order = $this->arguments['sort_order'];
		} else {
			$this->order = 'ASC';
		}

		if (isset($_GET['sort_field'])) {
			$this->field = $_GET['sort_field'];
			$this->model->merge_settings(array('sort_field' => $this->field));
		} elseif (isset($this->arguments['sort_field'])) {
			$this->field = $this->arguments['sort_field'];
		} else {
			$this->field = 'h.host_name';
		}

		// Rewrite old value that is often broken
		if ($this->field == 's.last_state_change' || $this->field == 'last_state_change') {
			$this->field = '-duration';
		}

		$this->state = 0;
		if ($this->arguments['critical'])
			$this->state |= nagstat::SERVICE_CRITICAL;
		if ($this->arguments['warning'])
			$this->state |= nagstat::SERVICE_WARNING;
		if ($this->arguments['unknown'])
			$this->state |= nagstat::SERVICE_UNKNOWN;

		$this->img_sort_up = $this->add_path('icons/16x16/up.gif');
		$this->img_sort_down = $this->add_path('icons/16x16/down.gif');

		# assign specific header fields and values for current method
		$header_link_fields = array(
			array('title' => _('Host'), 'sort_field_db' => 'h.host_name', 'sort_field_str' => 'host name'),
			array('title' => _('Status'), 'sort_field_db' => 's.current_state', 'sort_field_str' => 'service status'),
			array('title' => _('Service'), 'sort_field_db' => 's.service_description', 'sort_field_str' => 'service name'),
			array('title' => _('Last Check'), 'sort_field_db' => 'last_check', 'sort_field_str' => 'last check time'),
			array('title' => _('Alert time'), 'sort_field_db' => '-duration', 'sort_field_str' => 'alert time'),
			array('title' => _('Status Information'))
		);

		# build header links array
		foreach ($header_link_fields as $fields) {
			if (sizeof($fields) > 1) {
				$this->header_link_strings[] = $this->header_links('service', $this->arguments['group'], $fields['title'], Router::$method, $fields['sort_field_db'], $fields['sort_field_str']);
			} else {
				$this->header_link_strings[] = $this->header_links('service', $this->arguments['group'], $fields['title']);
			}
		}

		$this->host_comments = Comment_Model::count_comments_by_object();
		$this->comments = Comment_Model::count_comments_by_object(true);

		$this->js = array('application/widgets/tac_services_common/js/tac_services_common');
	}
}
